@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Parser."""
 import sys
 from .. import config
@@ -295,6 +315,14 @@ any spatial references. For further details, please check out \
 https://fmriprep.readthedocs.io/en/%s/spaces.html"""
         % (currentv.base_version if is_release else "latest"),
     )
+    g_conf.add_argument(
+        "--me-output-echos",
+        action="store_true",
+        default=False,
+        help="""\
+Output individual echo time series with slice, motion and susceptibility correction. \
+Useful for further Tedana processing post-fMRIPrep."""
+    )
 
     g_conf.add_argument(
         "--bold2t1w-init",
@@ -352,7 +380,7 @@ https://fmriprep.readthedocs.io/en/%s/spaces.html"""
         action="store",
         default=None,
         type=int,
-        help="Number of non steady state volumes.",
+        help="Number of nonsteady-state volumes.",
     )
     g_conf.add_argument(
         "--random-seed",
@@ -453,14 +481,25 @@ https://fmriprep.readthedocs.io/en/%s/spaces.html"""
         default=True,
         help="do not remove median (within mask) from fieldmap",
     )
+    g_fmap.add_argument(
+        "--topup-max-vols",
+        action="store",
+        default=5,
+        type=int,
+        help="maximum number of volumes to use with TOPUP, per-series (EPI or BOLD)",
+    )
 
     # SyN-unwarp options
     g_syn = parser.add_argument_group("Specific options for SyN distortion correction")
     g_syn.add_argument(
         "--use-syn-sdc",
-        action="store_true",
+        nargs="?",
+        choices=["warn", "error"],
+        action="store",
+        const="error",
         default=False,
-        help="EXPERIMENTAL: Use fieldmap-free distortion correction",
+        help="EXPERIMENTAL: Use fieldmap-free distortion correction; "
+             "if unable, error (default) or warn based on optional argument.",
     )
     g_syn.add_argument(
         "--force-syn",
@@ -518,12 +557,12 @@ https://fmriprep.readthedocs.io/en/%s/spaces.html"""
     g_other.add_argument(
         "--output-layout",
         action="store",
-        default="legacy",
+        default="bids",
         choices=("bids", "legacy"),
-        help="Organization of outputs. legacy (default) creates derivative "
-        "datasets as subdirectories of outputs. bids places fMRIPrep derivatives "
+        help="Organization of outputs. bids (default) places fMRIPrep derivatives "
         "directly in the output directory, and defaults to placing FreeSurfer "
-        "derivatives in <output-dir>/sourcedata/freesurfer."
+        "derivatives in <output-dir>/sourcedata/freesurfer. legacy creates "
+        "derivative datasets as subdirectories of outputs."
     )
     g_other.add_argument(
         "-w",
@@ -633,11 +672,11 @@ def parse_args(args=None, namespace=None):
 
     if opts.config_file:
         skip = {} if opts.reports_only else {"execution": ("run_uuid",)}
-        config.load(opts.config_file, skip=skip)
+        config.load(opts.config_file, skip=skip, init=False)
         config.loggers.cli.info(f"Loaded previous configuration file {opts.config_file}")
 
     config.execution.log_level = int(max(25 - 5 * opts.verbose_count, logging.DEBUG))
-    config.from_dict(vars(opts))
+    config.from_dict(vars(opts), init=['nipype'])
 
     if not config.execution.notrack:
         import pkgutil
@@ -724,6 +763,12 @@ applied."""
                 f"Could not clear all contents of working directory: {work_dir}"
             )
 
+    # Update the config with an empty dict to trigger initialization of all config
+    # sections (we used `init=False` above).
+    # This must be done after cleaning the work directory, or we could delete an
+    # open SQLite database
+    config.from_dict({})
+
     # Ensure input and output folders are not the same
     if output_dir == bids_dir:
         parser.error(
