@@ -1,5 +1,26 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """fMRI preprocessing workflow."""
 from .. import config
 
@@ -33,28 +54,16 @@ def main():
     # CRITICAL Call build_workflow(config_file, retval) in a subprocess.
     # Because Python on Linux does not ever free virtual memory (VM), running the
     # workflow construction jailed within a process preempts excessive VM buildup.
-    try:
-        with Manager() as mgr:
-            from .workflow import build_workflow
-
-            retval = mgr.dict()
-            p = Process(target=build_workflow, args=(str(config_file), retval))
-            p.start()
-            p.join()
-
-            retcode = p.exitcode or retval.get("return_code", 0)
-            fmriprep_wf = retval.get("workflow", None)
-    except ConnectionError:
-        config.loggers.workflow.warning(
-            "Could not build workflow in separate process. Attempting to build "
-            "in main process. This may impact memory footprint."
-        )
-        # Attempt to fall back to building in the main process
+    with Manager() as mgr:
         from .workflow import build_workflow
-        retval = {}
-        build_workflow(str(config_file), retval)
-        fmriprep_wf = retval["workflow"]
-        retcode = 0
+
+        retval = mgr.dict()
+        p = Process(target=build_workflow, args=(str(config_file), retval))
+        p.start()
+        p.join()
+
+        retcode = p.exitcode or retval.get("return_code", 0)
+        fmriprep_wf = retval.get("workflow", None)
 
     # CRITICAL Load the config from the file. This is necessary because the ``build_workflow``
     # function executed constrained in a process may change the config (and thus the global
@@ -104,16 +113,6 @@ def main():
     try:
         fmriprep_wf.run(**config.nipype.get_plugin())
     except Exception as e:
-        msg = "Issue encountered while running fMRIPrep."
-        from concurrent.futures.process import BrokenProcessPool
-        if isinstance(e, BrokenProcessPool):
-            msg += " Usually, this is a result of insufficent memory."
-            if config.environment.exec_env in ("docker", "fmriprep-docker"):
-                msg += (
-                    " Ensure Docker has sufficent resources available "
-                    "(https://docs.docker.com/docker-for-mac/#resources)"
-                )
-            config.loggers.workflow.error(msg)
         if not config.execution.notrack:
             from ..utils.sentry import process_crashfile
 
@@ -131,7 +130,7 @@ def main():
             if "Workflow did not execute cleanly" not in str(e):
                 sentry_sdk.capture_exception(e)
         config.loggers.workflow.critical("fMRIPrep failed: %s", e)
-        raise RuntimeError(msg) from e
+        raise
     else:
         config.loggers.workflow.log(25, "fMRIPrep finished successfully!")
         if not config.execution.notrack:
