@@ -1,7 +1,32 @@
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Test parser."""
 from packaging.version import Version
+from pkg_resources import resource_filename as pkgrf
+from argparse import ArgumentError
+from contextlib import nullcontext
 import pytest
-from ..parser import _build_parser
+from ..parser import _build_parser, parse_args
 from .. import version as _version
 from ... import config
 from ...tests.test_config import _reset_config
@@ -117,6 +142,20 @@ def test_get_parser_blacklist(monkeypatch, capsys, flagged):
         assert (flagged[1] or "reason: unknown") in captured
 
 
+def test_parse_args(tmp_path):
+    """Basic smoke test showing that our parse_args() function
+    implements the BIDS App protocol"""
+    bids_dir = pkgrf('fmriprep', 'data/tests/ds000005')
+    out_dir = tmp_path / "out"
+    work_dir = tmp_path / "work"
+
+    parse_args(args=[bids_dir, str(out_dir), "participant",  # BIDS App
+                     "-w", str(work_dir),                    # Don't pollute CWD
+                     "--skip-bids-validation"])              # Empty files make BIDS sad
+    assert config.execution.layout.root == bids_dir
+    _reset_config()
+
+
 def test_bids_filter_file(tmp_path, capsys):
     bids_path = tmp_path / "data"
     out_path = tmp_path / "out"
@@ -156,3 +195,31 @@ def test_slice_time_ref(tmp_path, st_ref):
 
     parser.parse_args(args)
     _reset_config()
+
+
+@pytest.mark.parametrize("args, expectation", (
+    ([], False),
+    (["--use-syn-sdc"], "error"),
+    (["--use-syn-sdc", "error"], "error"),
+    (["--use-syn-sdc", "warn"], "warn"),
+    (["--use-syn-sdc", "other"], (SystemExit, ArgumentError)),
+))
+def test_use_syn_sdc(tmp_path, args, expectation):
+    bids_path = tmp_path / "data"
+    out_path = tmp_path / "out"
+    args = [str(bids_path), str(out_path), "participant"] + args
+    bids_path.mkdir()
+
+    parser = _build_parser()
+
+    cm = nullcontext()
+    if isinstance(expectation, tuple):
+        cm = pytest.raises(expectation)
+
+    with cm:
+        opts = parser.parse_args(args)
+
+    if not isinstance(expectation, tuple):
+        assert opts.use_syn_sdc == expectation
+
+    _reset_config()
