@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 r"""
 A Python module to maintain unique, run-wide *fMRIPrep* settings.
 
@@ -189,7 +209,7 @@ except Exception:
 
 # Debug modes are names that influence the exposure of internal details to
 # the user, either through additional derivatives or increased verbosity
-DEBUG_MODES = ("compcor",)
+DEBUG_MODES = ("compcor", "fieldmaps")
 
 
 class _Config:
@@ -390,6 +410,8 @@ class execution(_Config):
     """Do not monitor *fMRIPrep* using Sentry.io."""
     output_dir = None
     """Folder where derivatives will be stored."""
+    me_output_echos = False
+    """Output individual echo time series with slice, motion and susceptibility correction"""
     output_layout = None
     """Layout of derivatives within output_dir."""
     output_spaces = None
@@ -437,15 +459,9 @@ class execution(_Config):
             from bids.layout.index import BIDSLayoutIndexer
             from bids.layout import BIDSLayout
 
-            if cls.bids_database_dir:
-                _db_path = cls.bids_database_dir
-                if not _db_path.exists():
-                    logging.getLogger("cli").warning(
-                        f"Creating PyBIDS database directory: {_db_path}"
-                    )
-            else:
-                _db_path = cls.work_dir / cls.run_uuid / "bids_db"
-
+            _db_path = cls.bids_database_dir or (
+                cls.work_dir / cls.run_uuid / "bids_db"
+            )
             _db_path.mkdir(exist_ok=True, parents=True)
 
             # Recommended after PyBIDS 12.1
@@ -553,6 +569,8 @@ class workflow(_Config):
     spaces = None
     """Keeps the :py:class:`~niworkflows.utils.spaces.SpatialReferences`
     instance keeping standard and nonstandard spaces."""
+    topup_max_vols = 5
+    """Maximum number of volumes to use with TOPUP, per-series (EPI or BOLD)."""
     use_aroma = None
     """Run ICA-:abbr:`AROMA (automatic removal of motion artifacts)`."""
     use_bbr = None
@@ -642,27 +660,58 @@ def _set_numpy_seed():
     return val
 
 
-def from_dict(settings):
-    """Read settings from a flat dictionary."""
-    nipype.load(settings)
-    execution.load(settings)
-    workflow.load(settings)
-    seeds.load(settings)
+def from_dict(settings, init=True, ignore=None):
+    """Read settings from a flat dictionary.
+
+    Arguments
+    ---------
+    setting : dict
+        Settings to apply to any configuration
+    init : `bool` or :py:class:`~collections.abc.Container`
+        Initialize all, none, or a subset of configurations.
+    ignore : :py:class:`~collections.abc.Container`
+        Collection of keys in ``setting`` to ignore
+    """
+
+    # Accept global True/False or container of configs to initialize
+    def initialize(x):
+        return init if init in (True, False) else x in init
+
+    nipype.load(settings, init=initialize('nipype'), ignore=ignore)
+    execution.load(settings, init=initialize('execution'), ignore=ignore)
+    workflow.load(settings, init=initialize('workflow'), ignore=ignore)
+    seeds.load(settings, init=initialize('seeds'), ignore=ignore)
+
     loggers.init()
 
 
-def load(filename, skip=None):
-    """Load settings from file."""
+def load(filename, skip=None, init=True):
+    """Load settings from file.
+
+    Arguments
+    ---------
+    filename : :py:class:`os.PathLike`
+        TOML file containing fMRIPrep configuration.
+    skip : dict or None
+        Sets of values to ignore during load, keyed by section name
+    init : `bool` or :py:class:`~collections.abc.Container`
+        Initialize all, none, or a subset of configurations.
+    """
     from toml import loads
 
     skip = skip or {}
+
+    # Accept global True/False or container of configs to initialize
+    def initialize(x):
+        return init if init in (True, False) else x in init
+
     filename = Path(filename)
     settings = loads(filename.read_text())
     for sectionname, configs in settings.items():
         if sectionname != "environment":
             section = getattr(sys.modules[__name__], sectionname)
             ignore = skip.get(sectionname)
-            section.load(configs, ignore=ignore)
+            section.load(configs, ignore=ignore, init=initialize(sectionname))
     init_spaces()
 
 
