@@ -1,7 +1,25 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """
 Multi-echo EPI
 ~~~~~~~~~~~~~~
@@ -38,9 +56,13 @@ class T2SMapInputSpec(CommandLineInputSpec):
                              mandatory=True,
                              minlen=3,
                              desc='echo times')
+    mask_file = File(argstr='--mask %s',
+                     position=3,
+                     desc='mask file',
+                     exists=True)
     fittype = traits.Enum('curvefit', 'loglin',
                           argstr='--fittype %s',
-                          position=3,
+                          position=4,
                           usedefault=True,
                           desc=('Desired fitting method: '
                                 '"loglin" means that a linear model is fit '
@@ -72,6 +94,7 @@ class T2SMap(CommandLine):
     >>> t2smap.cmdline  # doctest: +ELLIPSIS
     't2smap -d sub-01_run-01_echo-1_bold.nii.gz sub-01_run-01_echo-2_bold.nii.gz \
 sub-01_run-01_echo-3_bold.nii.gz -e 13.0 27.0 43.0 --fittype curvefit'
+
     """
     _cmd = 't2smap'
     input_spec = T2SMapInputSpec
