@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """
 fMRIPrep base processing workflows
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -17,7 +37,8 @@ from nipype.pipeline import engine as pe
 from nipype.interfaces import utility as niu
 
 from .. import config
-from ..interfaces import SubjectSummary, AboutSummary, DerivativesDataSink
+from ..interfaces import DerivativesDataSink
+from ..interfaces.reports import SubjectSummary, AboutSummary
 from .bold import init_func_preproc_wf
 
 
@@ -290,58 +311,204 @@ It is released under the [CC0]\
     if anat_only:
         return workflow
 
+    from sdcflows import fieldmaps as fm
+    fmap_estimators = None
+
+    if any(("fieldmaps" not in config.workflow.ignore,
+            config.workflow.use_syn_sdc,
+            config.workflow.force_syn)):
+        from sdcflows.utils.wrangler import find_estimators
+
+        # SDC Step 1: Run basic heuristics to identify available data for fieldmap estimation
+        # For now, no fmapless
+        fmap_estimators = find_estimators(
+            layout=config.execution.layout,
+            subject=subject_id,
+            fmapless=bool(config.workflow.use_syn_sdc),
+            force_fmapless=config.workflow.force_syn,
+        )
+
+        if config.workflow.use_syn_sdc and not fmap_estimators:
+            message = ("Fieldmap-less (SyN) estimation was requested, but "
+                       "PhaseEncodingDirection information appears to be "
+                       "absent.")
+            config.loggers.workflow.error(message)
+            if config.workflow.use_syn_sdc == "error":
+                raise ValueError(message)
+
+        if (
+            "fieldmaps" in config.workflow.ignore
+            and [f for f in fmap_estimators
+                 if f.method != fm.EstimatorType.ANAT]
+        ):
+            config.loggers.workflow.info(
+                'Option "--ignore fieldmaps" was set, but either "--use-syn-sdc" '
+                'or "--force-syn" were given, so fieldmap-less estimation will be executed.'
+            )
+            fmap_estimators = [f for f in fmap_estimators
+                               if f.method == fm.EstimatorType.ANAT]
+
+        if fmap_estimators:
+            config.loggers.workflow.info(
+                "B0 field inhomogeneity map will be estimated with "
+                f" the following {len(fmap_estimators)} estimators: "
+                f"{[e.method for e in fmap_estimators]}."
+            )
+
     # Append the functional section to the existing anatomical exerpt
     # That way we do not need to stream down the number of bold datasets
-    anat_preproc_wf.__postdesc__ = (anat_preproc_wf.__postdesc__ or '') + """
-
+    func_pre_desc = """
 Functional data preprocessing
 
 : For each of the {num_bold} BOLD runs found per subject (across all
 tasks and sessions), the following preprocessing was performed.
 """.format(num_bold=len(subject_data['bold']))
 
+    func_preproc_wfs = []
+    has_fieldmap = bool(fmap_estimators)
     for bold_file in subject_data['bold']:
-        func_preproc_wf = init_func_preproc_wf(bold_file)
+        func_preproc_wf = init_func_preproc_wf(bold_file, has_fieldmap=has_fieldmap)
         if func_preproc_wf is None:
             continue
 
-        try:
-            workflow.connect([
-                (anat_preproc_wf, func_preproc_wf,
-                 [('outputnode.t1w_preproc', 'inputnode.t1w_preproc'),
-                  ('outputnode.t1w_mask', 'inputnode.t1w_mask'),
-                  ('outputnode.t1w_dseg', 'inputnode.t1w_dseg'),
-                  ('outputnode.t1w_aseg', 'inputnode.t1w_aseg'),
-                  ('outputnode.t1w_aparc', 'inputnode.t1w_aparc'),
-                  ('outputnode.t1w_tpms', 'inputnode.t1w_tpms'),
-                  ('outputnode.template', 'inputnode.template'),
-                  ('outputnode.anat2std_xfm', 'inputnode.anat2std_xfm'),
-                  ('outputnode.std2anat_xfm', 'inputnode.std2anat_xfm'),
-                  # Undefined if --fs-no-reconall, but this is safe
-                  ('outputnode.subjects_dir', 'inputnode.subjects_dir'),
-                  ('outputnode.subject_id', 'inputnode.subject_id'),
-                  ('outputnode.t1w2fsnative_xfm', 'inputnode.t1w2fsnative_xfm'),
-                  ('outputnode.fsnative2t1w_xfm', 'inputnode.fsnative2t1w_xfm')]),
-            ])
-        except OSError as err:
-            if err.args[0].startswith("Duplicate node name"):
-                msg = (
-                    f"Could not create subworkflow for {bold_file}. "
-                    f"{func_preproc_wf.name} already exists. "
-                    "Check if uncompressed (.nii) and compressed (.nii.gz) files are both present."
-                )
-                config.loggers.workflow.error(msg)
-                raise OSError(msg) from err
+        func_preproc_wf.__desc__ = func_pre_desc + (func_preproc_wf.__desc__ or "")
+        workflow.connect([
+            (anat_preproc_wf, func_preproc_wf,
+             [('outputnode.t1w_preproc', 'inputnode.t1w_preproc'),
+              ('outputnode.t1w_mask', 'inputnode.t1w_mask'),
+              ('outputnode.t1w_dseg', 'inputnode.t1w_dseg'),
+              ('outputnode.t1w_aseg', 'inputnode.t1w_aseg'),
+              ('outputnode.t1w_aparc', 'inputnode.t1w_aparc'),
+              ('outputnode.t1w_tpms', 'inputnode.t1w_tpms'),
+              ('outputnode.template', 'inputnode.template'),
+              ('outputnode.anat2std_xfm', 'inputnode.anat2std_xfm'),
+              ('outputnode.std2anat_xfm', 'inputnode.std2anat_xfm'),
+              # Undefined if --fs-no-reconall, but this is safe
+              ('outputnode.subjects_dir', 'inputnode.subjects_dir'),
+              ('outputnode.subject_id', 'inputnode.subject_id'),
+              ('outputnode.t1w2fsnative_xfm', 'inputnode.t1w2fsnative_xfm'),
+              ('outputnode.fsnative2t1w_xfm', 'inputnode.fsnative2t1w_xfm')]),
+        ])
+        func_preproc_wfs.append(func_preproc_wf)
+
+    if not has_fieldmap:
+        return workflow
+
+    from sdcflows.workflows.base import init_fmap_preproc_wf
+
+    fmap_wf = init_fmap_preproc_wf(
+        debug="fieldmaps" in config.execution.debug,
+        estimators=fmap_estimators,
+        omp_nthreads=config.nipype.omp_nthreads,
+        output_dir=fmriprep_dir,
+        subject=subject_id,
+    )
+    fmap_wf.__desc__ = f"""
+
+Preprocessing of B<sub>0</sub> inhomogeneity mappings
+
+: A total of {len(fmap_estimators)} fieldmaps were found available within the input
+BIDS structure for this particular subject.
+"""
+    for func_preproc_wf in func_preproc_wfs:
+        # fmt: off
+        workflow.connect([
+            (fmap_wf, func_preproc_wf, [
+                ("outputnode.fmap", "inputnode.fmap"),
+                ("outputnode.fmap_ref", "inputnode.fmap_ref"),
+                ("outputnode.fmap_coeff", "inputnode.fmap_coeff"),
+                ("outputnode.fmap_mask", "inputnode.fmap_mask"),
+                ("outputnode.fmap_id", "inputnode.fmap_id"),
+                ("outputnode.method", "inputnode.sdc_method"),
+            ]),
+        ])
+        # fmt: on
+
+    # Overwrite ``out_path_base`` of sdcflows's DataSinks
+    for node in fmap_wf.list_node_names():
+        if node.split(".")[-1].startswith("ds_"):
+            fmap_wf.get_node(node).interface.out_path_base = ""
+
+    # Step 3: Manually connect PEPOLAR and ANAT workflows
+
+    # Select "MNI152NLin2009cAsym" from standard references.
+    # This node may be used by multiple ANAT estimators, so define outside loop.
+    from niworkflows.interfaces.utility import KeySelect
+    fmap_select_std = pe.Node(
+        KeySelect(fields=["std2anat_xfm"], key="MNI152NLin2009cAsym"),
+        name="fmap_select_std",
+        run_without_submitting=True,
+    )
+    if any(estimator.method == fm.EstimatorType.ANAT for estimator in fmap_estimators):
+        # fmt:off
+        workflow.connect([
+            (anat_preproc_wf, fmap_select_std, [
+                ("outputnode.std2anat_xfm", "std2anat_xfm"),
+                ("outputnode.template", "keys")]),
+        ])
+        # fmt:on
+
+    for estimator in fmap_estimators:
+        config.loggers.workflow.info(f"""\
+Setting-up fieldmap "{estimator.bids_id}" ({estimator.method}) with \
+<{', '.join(s.path.name for s in estimator.sources)}>""")
+
+        # Mapped and phasediff can be connected internally by SDCFlows
+        if estimator.method in (fm.EstimatorType.MAPPED, fm.EstimatorType.PHASEDIFF):
+            continue
+
+        suffices = [s.suffix for s in estimator.sources]
+
+        if estimator.method == fm.EstimatorType.PEPOLAR:
+            if set(suffices) == {"epi"} or sorted(suffices) == ["bold", "epi"]:
+                wf_inputs = getattr(fmap_wf.inputs, f"in_{estimator.bids_id}")
+                wf_inputs.in_data = [str(s.path) for s in estimator.sources]
+                wf_inputs.metadata = [s.metadata for s in estimator.sources]
+
+                # 21.0.x hack to change the number of volumes used
+                # The default of 50 takes excessively long
+                flatten = fmap_wf.get_node(f"wf_{estimator.bids_id}.flatten")
+                flatten.inputs.max_trs = config.workflow.topup_max_vols
             else:
-                raise
+                raise NotImplementedError(
+                    "Sophisticated PEPOLAR schemes are unsupported."
+                )
+
+        elif estimator.method == fm.EstimatorType.ANAT:
+            from sdcflows.workflows.fit.syn import init_syn_preprocessing_wf
+
+            sources = [str(s.path) for s in estimator.sources if s.suffix == "bold"]
+            source_meta = [s.metadata for s in estimator.sources if s.suffix == "bold"]
+            syn_preprocessing_wf = init_syn_preprocessing_wf(
+                omp_nthreads=config.nipype.omp_nthreads,
+                debug=config.execution.sloppy,
+                auto_bold_nss=True,
+                t1w_inversion=False,
+                name=f"syn_preprocessing_{estimator.bids_id}",
+            )
+            syn_preprocessing_wf.inputs.inputnode.in_epis = sources
+            syn_preprocessing_wf.inputs.inputnode.in_meta = source_meta
+
+            # fmt:off
+            workflow.connect([
+                (anat_preproc_wf, syn_preprocessing_wf, [
+                    ("outputnode.t1w_preproc", "inputnode.in_anat"),
+                    ("outputnode.t1w_mask", "inputnode.mask_anat"),
+                ]),
+                (fmap_select_std, syn_preprocessing_wf, [
+                    ("std2anat_xfm", "inputnode.std2anat_xfm"),
+                ]),
+                (syn_preprocessing_wf, fmap_wf, [
+                    ("outputnode.epi_ref", f"in_{estimator.bids_id}.epi_ref"),
+                    ("outputnode.epi_mask", f"in_{estimator.bids_id}.epi_mask"),
+                    ("outputnode.anat_ref", f"in_{estimator.bids_id}.anat_ref"),
+                    ("outputnode.anat_mask", f"in_{estimator.bids_id}.anat_mask"),
+                    ("outputnode.sd_prior", f"in_{estimator.bids_id}.sd_prior"),
+                ]),
+            ])
+            # fmt:on
     return workflow
 
 
 def _prefix(subid):
     return subid if subid.startswith('sub-') else f'sub-{subid}'
-
-
-def _pop(inlist):
-    if isinstance(inlist, (list, tuple)):
-        return inlist[0]
-    return inlist
