@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Writing out derivative files."""
 import numpy as np
 from nipype.pipeline import engine as pe
@@ -96,7 +116,8 @@ def init_func_derivatives_wf(
     bids_root,
     cifti_output,
     freesurfer,
-    metadata,
+    all_metadata,
+    multiecho,
     output_dir,
     spaces,
     use_aroma,
@@ -115,6 +136,8 @@ def init_func_derivatives_wf(
         Whether FreeSurfer anatomical processing was run.
     metadata : :obj:`dict`
         Metadata dictionary associated to the BOLD run.
+    multiecho : :obj:`bool`
+        Derivatives were generated from multi-echo time series.
     output_dir : :obj:`str`
         Where derivatives should be written out to.
     spaces : :py:class:`~niworkflows.utils.spaces.SpatialReferences`
@@ -136,18 +159,26 @@ def init_func_derivatives_wf(
     from niworkflows.interfaces.utility import KeySelect
     from smriprep.workflows.outputs import _bids_relative
 
+    metadata = all_metadata[0]
+
     timing_parameters = prepare_timing_parameters(metadata)
 
     nonstd_spaces = set(spaces.get_nonstandard())
     workflow = Workflow(name=name)
 
+    # BOLD series will generally be unmasked unless multiecho,
+    # as the optimal combination is undefined outside a bounded mask
+    masked = multiecho
+
     inputnode = pe.Node(niu.IdentityInterface(fields=[
         'aroma_noise_ics', 'bold_aparc_std', 'bold_aparc_t1', 'bold_aseg_std',
         'bold_aseg_t1', 'bold_cifti', 'bold_mask_std', 'bold_mask_t1', 'bold_std',
         'bold_std_ref', 'bold_t1', 'bold_t1_ref', 'bold_native', 'bold_native_ref',
-        'bold_mask_native', 'cifti_variant', 'cifti_metadata', 'cifti_density',
+        'bold_mask_native', 'bold_echos_native',
+        'cifti_variant', 'cifti_metadata', 'cifti_density',
         'confounds', 'confounds_metadata', 'melodic_mix', 'nonaggr_denoised_file',
-        'source_file', 'surf_files', 'surf_refs', 'template', 'spatial_reference',
+        'source_file', 'all_source_files',
+        'surf_files', 'surf_refs', 'template', 'spatial_reference',
         'bold2anat_xfm', 'anat2bold_xfm', 'acompcor_masks', 'tcompcor_mask']),
         name='inputnode')
 
@@ -175,7 +206,7 @@ def init_func_derivatives_wf(
         name='ds_t1w_tpl_inv_xfm', run_without_submitting=True)
 
     workflow.connect([
-        (inputnode, raw_sources, [('source_file', 'in_files')]),
+        (inputnode, raw_sources, [('all_source_files', 'in_files')]),
         (inputnode, ds_confounds, [('source_file', 'source_file'),
                                    ('confounds', 'in_file'),
                                    ('confounds_metadata', 'meta_dict')]),
@@ -185,13 +216,22 @@ def init_func_derivatives_wf(
                                          ('anat2bold_xfm', 'in_file')]),
     ])
 
-    if nonstd_spaces.intersection(('func', 'run', 'bold', 'boldref', 'sbref')):
+    bold_output = nonstd_spaces.intersection(('func', 'run', 'bold', 'boldref', 'sbref'))
+    if bold_output:
         ds_bold_native = pe.Node(
             DerivativesDataSink(
-                base_directory=output_dir, desc='preproc', compress=True, SkullStripped=False,
+                base_directory=output_dir, desc='preproc', compress=True, SkullStripped=masked,
                 TaskName=metadata.get('TaskName'), **timing_parameters),
             name='ds_bold_native', run_without_submitting=True,
             mem_gb=DEFAULT_MEMORY_MIN_GB)
+        # fmt: off
+        workflow.connect([
+            (inputnode, ds_bold_native, [('source_file', 'source_file'),
+                                         ('bold_native', 'in_file')]),
+        ])
+        # fmt:on
+    # Save masks and boldref if we're going to save either orig BOLD series or echos
+    if bold_output or multiecho and config.execution.me_output_echos:
         ds_bold_native_ref = pe.Node(
             DerivativesDataSink(base_directory=output_dir, suffix='boldref', compress=True,
                                 dismiss_entities=("echo",)),
@@ -204,8 +244,6 @@ def init_func_derivatives_wf(
             mem_gb=DEFAULT_MEMORY_MIN_GB)
 
         workflow.connect([
-            (inputnode, ds_bold_native, [('source_file', 'source_file'),
-                                         ('bold_native', 'in_file')]),
             (inputnode, ds_bold_native_ref, [('source_file', 'source_file'),
                                              ('bold_native_ref', 'in_file')]),
             (inputnode, ds_bold_mask_native, [('source_file', 'source_file'),
@@ -213,12 +251,30 @@ def init_func_derivatives_wf(
             (raw_sources, ds_bold_mask_native, [('out', 'RawSources')]),
         ])
 
+    if multiecho and config.execution.me_output_echos:
+        ds_bold_echos_native = pe.MapNode(
+            DerivativesDataSink(
+                base_directory=output_dir, desc='preproc', compress=True, SkullStripped=False,
+                TaskName=metadata.get('TaskName'), **timing_parameters),
+            iterfield=['source_file', 'in_file', 'meta_dict'],
+            name='ds_bold_echos_native', run_without_submitting=True,
+            mem_gb=DEFAULT_MEMORY_MIN_GB)
+        ds_bold_echos_native.inputs.meta_dict = [
+            {"EchoTime": md["EchoTime"]} for md in all_metadata
+        ]
+
+        workflow.connect([
+            (inputnode, ds_bold_echos_native, [
+                ('all_source_files', 'source_file'),
+                ('bold_echos_native', 'in_file')]),
+        ])
+
     # Resample to T1w space
     if nonstd_spaces.intersection(('T1w', 'anat')):
         ds_bold_t1 = pe.Node(
             DerivativesDataSink(
                 base_directory=output_dir, space='T1w', desc='preproc', compress=True,
-                SkullStripped=False, TaskName=metadata.get('TaskName'), **timing_parameters),
+                SkullStripped=masked, TaskName=metadata.get('TaskName'), **timing_parameters),
             name='ds_bold_t1', run_without_submitting=True,
             mem_gb=DEFAULT_MEMORY_MIN_GB)
         ds_bold_t1_ref = pe.Node(
@@ -303,7 +359,7 @@ def init_func_derivatives_wf(
 
         ds_bold_std = pe.Node(
             DerivativesDataSink(
-                base_directory=output_dir, desc='preproc', compress=True, SkullStripped=False,
+                base_directory=output_dir, desc='preproc', compress=True, SkullStripped=masked,
                 TaskName=metadata.get('TaskName'), **timing_parameters),
             name='ds_bold_std', run_without_submitting=True, mem_gb=DEFAULT_MEMORY_MIN_GB)
         ds_bold_std_ref = pe.Node(
@@ -480,7 +536,7 @@ def init_bold_preproc_report_wf(mem_gb, reportlets_dir, name='bold_preproc_repor
     """
     from nipype.algorithms.confounds import TSNR
     from niworkflows.engine.workflows import LiterateWorkflow as Workflow
-    from niworkflows.interfaces import SimpleBeforeAfter
+    from niworkflows.interfaces.reportlets.registration import SimpleBeforeAfterRPT
     from ...interfaces import DerivativesDataSink
 
     workflow = Workflow(name=name)
@@ -491,7 +547,7 @@ def init_bold_preproc_report_wf(mem_gb, reportlets_dir, name='bold_preproc_repor
     pre_tsnr = pe.Node(TSNR(), name='pre_tsnr', mem_gb=mem_gb * 4.5)
     pos_tsnr = pe.Node(TSNR(), name='pos_tsnr', mem_gb=mem_gb * 4.5)
 
-    bold_rpt = pe.Node(SimpleBeforeAfter(), name='bold_rpt',
+    bold_rpt = pe.Node(SimpleBeforeAfterRPT(), name='bold_rpt',
                        mem_gb=0.1)
     ds_report_bold = pe.Node(
         DerivativesDataSink(base_directory=reportlets_dir, desc='preproc',
