@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """
 Registration workflows
 ++++++++++++++++++++++
@@ -140,7 +160,8 @@ def init_bold_reg_wf(
                                bold2t1w_init=bold2t1w_init, omp_nthreads=omp_nthreads)
     else:
         bbr_wf = init_fsl_bbr_wf(use_bbr=use_bbr, bold2t1w_dof=bold2t1w_dof,
-                                 bold2t1w_init=bold2t1w_init, sloppy=sloppy)
+                                 bold2t1w_init=bold2t1w_init, sloppy=sloppy,
+                                 omp_nthreads=omp_nthreads)
 
     workflow.connect([
         (inputnode, bbr_wf, [
@@ -255,12 +276,13 @@ def init_bold_t1_trans_wf(freesurfer, mem_gb, omp_nthreads, use_compression=True
       * :py:func:`~fmriprep.workflows.bold.registration.init_fsl_bbr_wf`
 
     """
+    from fmriprep.interfaces.maths import Clip
     from niworkflows.engine.workflows import LiterateWorkflow as Workflow
     from niworkflows.func.util import init_bold_reference_wf
     from niworkflows.interfaces.fixes import FixHeaderApplyTransforms as ApplyTransforms
     from niworkflows.interfaces.itk import MultiApplyTransforms
     from niworkflows.interfaces.nilearn import Merge
-    from niworkflows.interfaces.utils import GenerateSamplingReference
+    from niworkflows.interfaces.nibabel import GenerateSamplingReference
 
     workflow = Workflow(name=name)
     inputnode = pe.Node(
@@ -317,6 +339,13 @@ def init_bold_t1_trans_wf(freesurfer, mem_gb, omp_nthreads, use_compression=True
         MultiApplyTransforms(interpolation="LanczosWindowedSinc", float=True, copy_dtype=True),
         name='bold_to_t1w_transform', mem_gb=mem_gb * 3 * omp_nthreads, n_procs=omp_nthreads)
 
+    # Interpolation can occasionally produce below-zero values as an artifact
+    threshold = pe.MapNode(
+        Clip(minimum=0),
+        name="threshold",
+        iterfield=['in_file'],
+        mem_gb=DEFAULT_MEMORY_MIN_GB)
+
     # merge 3D volumes into 4D timeseries
     merge = pe.Node(Merge(compress=use_compression), name='merge', mem_gb=mem_gb)
 
@@ -336,7 +365,8 @@ def init_bold_t1_trans_wf(freesurfer, mem_gb, omp_nthreads, use_compression=True
         (inputnode, bold_to_t1w_transform, [('bold_split', 'input_image')]),
         (merge_xforms, bold_to_t1w_transform, [('out', 'transforms')]),
         (gen_ref, bold_to_t1w_transform, [('out_file', 'reference_image')]),
-        (bold_to_t1w_transform, merge, [('out_files', 'in_files')]),
+        (bold_to_t1w_transform, threshold, [('out_files', 'in_file')]),
+        (threshold, merge, [('out_file', 'in_files')]),
         (merge, gen_final_ref, [('out_file', 'inputnode.bold_file')]),
         (mask_t1w_tfm, gen_final_ref, [('output_image', 'inputnode.bold_mask')]),
         (merge, outputnode, [('out_file', 'bold_t1')]),
@@ -559,7 +589,8 @@ Co-registration was configured with {dof} degrees of freedom{reason}.
     return workflow
 
 
-def init_fsl_bbr_wf(use_bbr, bold2t1w_dof, bold2t1w_init, sloppy=False, name='fsl_bbr_wf'):
+def init_fsl_bbr_wf(use_bbr, bold2t1w_dof, bold2t1w_init, omp_nthreads, sloppy=False,
+                    name='fsl_bbr_wf'):
     """
     Build a workflow to run FSL's ``flirt``.
 
@@ -628,16 +659,21 @@ def init_fsl_bbr_wf(use_bbr, bold2t1w_dof, bold2t1w_init, sloppy=False, name='fs
     """
     from niworkflows.engine.workflows import LiterateWorkflow as Workflow
     from niworkflows.utils.images import dseg_label as _dseg_label
-    from niworkflows.interfaces.freesurfer import PatchedLTAConvert as LTAConvert
-    from niworkflows.interfaces.registration import FLIRTRPT
+    from niworkflows.interfaces.freesurfer import (
+        PatchedLTAConvert as LTAConvert,
+        PatchedMRICoregRPT as MRICoregRPT,
+    )
+    from niworkflows.interfaces.reportlets.registration import FLIRTRPT
     workflow = Workflow(name=name)
     workflow.__desc__ = """\
 The BOLD reference was then co-registered to the T1w reference using
-`flirt` [FSL {fsl_ver}, @flirt] with the boundary-based registration [@bbr]
-cost-function.
-Co-registration was configured with nine degrees of freedom to account
-for distortions remaining in the BOLD reference.
-""".format(fsl_ver=FLIRTRPT().version or '<ver>')
+`mri_coreg` (FreeSurfer) followed by `flirt` [FSL {fsl_ver}, @flirt]
+with the boundary-based registration [@bbr] cost-function.
+Co-registration was configured with {dof} degrees of freedom{reason}.
+""".format(fsl_ver=FLIRTRPT().version or '<ver>',
+           dof={6: 'six', 9: 'nine', 12: 'twelve'}[bold2t1w_dof],
+           reason='' if bold2t1w_dof == 6 else
+                  'to account for distortions remaining in the BOLD reference')
 
     inputnode = pe.Node(
         niu.IdentityInterface([
@@ -651,9 +687,6 @@ for distortions remaining in the BOLD reference.
 
     wm_mask = pe.Node(niu.Function(function=_dseg_label), name='wm_mask')
     wm_mask.inputs.label = 2  # BIDS default is WM=2
-    flt_bbr_init = pe.Node(FLIRTRPT(dof=6, generate_report=not use_bbr,
-                                    uses_qform=True, args="-basescale 1"),
-                           name='flt_bbr_init')
 
     if bold2t1w_init not in ("register", "header"):
         raise ValueError(f"Unknown BOLD-T1w initialization option: {bold2t1w_init}")
@@ -661,6 +694,17 @@ for distortions remaining in the BOLD reference.
     if bold2t1w_init == "header":
         raise NotImplementedError("Header-based registration initialization not supported for FSL")
 
+    mri_coreg = pe.Node(
+        MRICoregRPT(dof=bold2t1w_dof, sep=[4], ftol=0.0001, linmintol=0.01,
+                    generate_report=not use_bbr),
+        name='mri_coreg', n_procs=omp_nthreads, mem_gb=5)
+
+    lta_to_fsl = pe.Node(LTAConvert(out_fsl=True), name='lta_to_fsl',
+                         mem_gb=DEFAULT_MEMORY_MIN_GB)
+    workflow.connect([
+        (mri_coreg, lta_to_fsl, [('out_lta_file', 'in_lta')]),
+    ])
+
     invt_bbr = pe.Node(fsl.ConvertXFM(invert_xfm=True), name='invt_bbr',
                        mem_gb=DEFAULT_MEMORY_MIN_GB)
 
@@ -672,8 +716,8 @@ for distortions remaining in the BOLD reference.
                           name='fsl2itk_inv', mem_gb=DEFAULT_MEMORY_MIN_GB)
 
     workflow.connect([
-        (inputnode, flt_bbr_init, [('in_file', 'in_file'),
-                                   ('t1w_brain', 'reference')]),
+        (inputnode, mri_coreg, [('in_file', 'source_file'),
+                                ('t1w_brain', 'reference_file')]),
         (inputnode, fsl2itk_fwd, [('t1w_brain', 'reference_file'),
                                   ('in_file', 'source_file')]),
         (inputnode, fsl2itk_inv, [('in_file', 'reference_file'),
@@ -686,9 +730,9 @@ for distortions remaining in the BOLD reference.
     # Short-circuit workflow building, use rigid registration
     if use_bbr is False:
         workflow.connect([
-            (flt_bbr_init, invt_bbr, [('out_matrix_file', 'in_file')]),
-            (flt_bbr_init, fsl2itk_fwd, [('out_matrix_file', 'transform_file')]),
-            (flt_bbr_init, outputnode, [('out_report', 'out_report')]),
+            (lta_to_fsl, invt_bbr, [('out_fsl', 'in_file')]),
+            (lta_to_fsl, fsl2itk_fwd, [('out_fsl', 'transform_file')]),
+            (mri_coreg, outputnode, [('out_report', 'out_report')]),
         ])
         outputnode.inputs.fallback = True
 
@@ -709,7 +753,7 @@ for distortions remaining in the BOLD reference.
     workflow.connect([
         (inputnode, wm_mask, [('t1w_dseg', 'in_seg')]),
         (inputnode, flt_bbr, [('in_file', 'in_file')]),
-        (flt_bbr_init, flt_bbr, [('out_matrix_file', 'in_matrix_file')]),
+        (lta_to_fsl, flt_bbr, [('out_fsl', 'in_matrix_file')]),
     ])
 
     if sloppy is True:
@@ -752,7 +796,7 @@ for distortions remaining in the BOLD reference.
 
     workflow.connect([
         (flt_bbr, transforms, [('out_matrix_file', 'in1')]),
-        (flt_bbr_init, transforms, [('out_matrix_file', 'in2')]),
+        (lta_to_fsl, transforms, [('out_fsl', 'in2')]),
         # Convert FSL transforms to LTA (RAS2RAS) transforms and compare
         (inputnode, fsl_to_lta, [('in_file', 'source_file'),
                                  ('t1w_brain', 'target_file')]),
@@ -765,7 +809,7 @@ for distortions remaining in the BOLD reference.
         (select_transform, invt_bbr, [('out', 'in_file')]),
         (select_transform, fsl2itk_fwd, [('out', 'transform_file')]),
         (flt_bbr, reports, [('out_report', 'in1')]),
-        (flt_bbr_init, reports, [('out_report', 'in2')]),
+        (mri_coreg, reports, [('out_report', 'in2')]),
         (reports, select_report, [('out', 'inlist')]),
         (compare_transforms, select_report, [('out', 'index')]),
         (select_report, outputnode, [('out', 'out_report')]),
