@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """
 Slice-Timing Correction (STC) of BOLD images
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -8,8 +28,10 @@ Slice-Timing Correction (STC) of BOLD images
 
 """
 import numpy as np
+import nibabel as nb
 from nipype.pipeline import engine as pe
 from nipype.interfaces import utility as niu, afni
+from nipype.interfaces.base import isdefined
 
 from ... import config
 
@@ -17,6 +39,20 @@ from ... import config
 LOGGER = config.loggers.workflow
 
 
+class TShift(afni.TShift):
+    """Patched version of TShift implementing the "TooShort" behavior."""
+
+    def _pre_run_hook(self, runtime):
+        ignore = self.inputs.ignore if isdefined(self.inputs.ignore) else 0
+        ntsteps = nb.load(self.inputs.in_file).shape[3]
+        if ntsteps - ignore < 5:
+            raise RuntimeError(
+                f"Insufficient length of BOLD data ({ntsteps} time points) after "
+                f"discarding {ignore} nonsteady-state (or 'dummy') time points."
+            )
+        return runtime
+
+
 def init_bold_stc_wf(metadata, name='bold_stc_wf'):
     """
     Create a workflow for :abbr:`STC (slice-timing correction)`.
@@ -56,7 +92,7 @@ def init_bold_stc_wf(metadata, name='bold_stc_wf'):
 
     """
     from niworkflows.engine.workflows import LiterateWorkflow as Workflow
-    from niworkflows.interfaces.utils import CopyXForm
+    from niworkflows.interfaces.header import CopyXForm
 
     slice_times = metadata["SliceTiming"]
     first, last = min(slice_times), max(slice_times)
@@ -76,11 +112,11 @@ BOLD runs were slice-time corrected to {tzero:0.3g}s ({frac:g} of slice acquisit
 
     # It would be good to fingerprint memory use of afni.TShift
     slice_timing_correction = pe.Node(
-        afni.TShift(outputtype='NIFTI_GZ',
-                    tr='{}s'.format(metadata["RepetitionTime"]),
-                    slice_timing=metadata['SliceTiming'],
-                    slice_encoding_direction=metadata.get('SliceEncodingDirection', 'k'),
-                    tzero=tzero),
+        TShift(outputtype='NIFTI_GZ',
+               tr=f"{metadata['RepetitionTime']}s",
+               slice_timing=metadata['SliceTiming'],
+               slice_encoding_direction=metadata.get('SliceEncodingDirection', 'k'),
+               tzero=tzero),
         name='slice_timing_correction')
 
     copy_xform = pe.Node(CopyXForm(), name='copy_xform', mem_gb=0.1)
